#       
#	n2 --- n0 -------------- n1 --- n3
#     
# Crea un instancia del simulador (creando el despachador de tareas)

set ns [new Simulator]


# Crea un archivo de salida
set salidaNAM [open salidaTCP.nam w]

# Registra todos los eventos en el archivo creado, utilizando el formato del NAM
$ns namtrace-all $salidaNAM

# Crea un archivo de salida donde guardar los datos asociados a la cola del enlace n0-n1  
set cola01 [open salidaCola01.tr w]

# Crea un archivo de salida donde registrar el tamao de la ventana del nodo n2
set ventana2 [open salidaVentana2.tr w]

# Crea un archivo de salida para monitorear el rtt y otro para el rtt promedio ( estos archivos se visulizarn con xgraph)
set trace_rtt [open salidaRTT.tr w]
set trace_srtt [open salidaRTTAvg.tr w] 

# Crea archivos de salida para monitorear el nmero mximo de bytes enviados por el nodo n2
set trace_seq2 [open salidaSeq2.tr w]

# Crea archivos de salida para monitorear los paquetes perdidos y arribados en el enlace crtico
set trace_perdidos [open paquetesPerdidos.tr w]
set trace_arribos  [open paquetesArribados.tr w]


# Crea un archivo para monitorear la cola y ver su tamao con Xgraph
set fq [open tamCola01.tr w]

# Define los cdigos de colores
$ns color 1 SeaGreen
$ns color 2 Red

for {set i 0} {$i < 4} {incr i} {
                set n($i) [$ns node]
}


# Crea el enlace n0-n1 ser el cuello de botella,  el enlace crtico es Drop Tail 
$ns duplex-link $n(0) $n(1) 1Mb 100ms DropTail

# Establece la ubicacin en el dibujo (nam)             
$ns duplex-link-op $n(0) $n(1) orient right


# Se va a monitorear la cola del enlace cuello de botella n0-n1, se crea un monitor de la cola
$ns trace-queue $n(0) $n(1) 
set qmon1 [$ns monitor-queue $n(0) $n(1) ""]

# Se establece el tamao del buffer de la cola del cuello de botella (en paquetes). Se debe descontar 1 (paquete en servicio)
$ns queue-limit $n(0) $n(1) 5

# Monitorea la cola del enlace n0-n1 (para visualizar en el NAM)
$ns duplex-link-op $n(0) $n(1) queuePos 0.5

# Crea enlaces entre el nodo fuente y el nodo n0 

$ns duplex-link  $n(2) $n(0) 2Mb 1ms  DropTail
$ns duplex-link-op $n(2) $n(0) orient right

# Crea enlaces entre el nodo n1 y el nodo n3 

$ns duplex-link $n(1)  $n(3)  2Mb 1ms DropTail
$ns duplex-link-op $n(1)  $n(3)  orient right

# Configura las variable de TCP que interesan, en las otras se usan valores por defecto


Agent/TCP set nam_tracevar_ true        
Agent/TCP set window_ 200

# TCP RENO entre los nodos n2 y n3

    set tcp1 [new Agent/TCP/Newreno]
    $ns attach-agent $n(2) $tcp1
    
    set sink(3) [new Agent/TCPSink]
    $ns attach-agent $n(3) $sink(3)

# Conecta la fuente y el destino
    $ns connect $tcp1 $sink(3)

# Crea aplicaciones FTP que las conecta a los nodos TCP creados
    set ftp1 [new Application/FTP]
    $ftp1 attach-agent $tcp1

# Declara las variables que se monitoraran de tcp para el nodo 2   
    $ns add-agent-trace $tcp1 tcp
    $ns monitor-agent-trace $tcp1
    $tcp1 tracevar cwnd_
    $tcp1 tracevar ssthresh_
    $tcp1 tracevar maxseq_         ;# nmero se secuencia.
    $tcp1 tracevar ack_            ;# nmero de ACK.
    $tcp1 tracevar dupacks_        ;# contador de ACKs.
    $tcp1 tracevar rtt_
    $tcp1 set fid_ 1
    $tcp1 set TCPTick 0.01

# Inicializa una serie de variables que utilizar para clculos

set avgrtt  0
set count 0
set totrtt 0
set recibidos 0
set recibidos_ant 0
set ventana 0


# Crea un procedimiento que se llama al arrancar y se autollama peridicamente

proc record {} {
         global ns tcp1 trace_rtt trace_srtt avgrtt count totrtt qmon1 aux_rec recibidos_ant ventana2 trace_ventanaMax trace_ventanaAvg Cantidad redq fq trace_seq2  trace_perdidos trace_arribos
         set count [expr $count +1]    
         set time 0.01
         set now [$ns now]

         #carga en la variable ventana el valor de la ventana actual de tcp y registra su valor en el archivo de salida
	 set ventana [$tcp1 set cwnd_]
	 puts $ventana2 "$now $ventana"

         #Se registra el numero mximo de paquetes enviado
         set seq2 [$tcp1 set maxseq_]
	 puts $trace_seq2 "$now $seq2"

	 # Se asume que TCPTick es 0.01, calcula el rtt y lo registra
	 set curr_rtt [$tcp1 set rtt_] 
         set curr_rtt [expr $curr_rtt * 0.01] 
	 puts $trace_rtt "$now $curr_rtt"        

	 #Calcula el rtt promedio
         set totrtt [expr $totrtt+$curr_rtt]
         set avgrtt [expr $totrtt/$count]     
         puts $trace_srtt "$now $avgrtt"

	 #registra el tamao de la cola
  	 puts $fq "$now [eval $qmon1 set size_] "
         set arribos  [$qmon1 set parrivals_]
         set perdidos [$qmon1 set pdrops_]
         puts $trace_arribos "$now $arribos"
         puts $trace_perdidos "$now $perdidos"
                 
	 #registra para que este procedimiento se vuelva a correr 
	 $ns at [expr $now+$time] "record"
}

proc fin {} {
    global ns cola01 qmon1 trace_rtt trace_srtt redf trace_arribos trace_perdidos ventana2 trace_seq2
	
	# Habilitarlo solo cuando hay pocas fuentes
	
	$ns flush-trace
	puts $cola01 [format "Recibidos: %7d (pkts) " [eval $qmon1 set parrivals_] ]
	puts $cola01 [format "Perdidos : %7d (pkts) %10d (bytes)" [eval $qmon1 set pdrops_] [eval $qmon1 set bdrops_]]
	close $cola01
        
        # Habilitarlo solo cuando hay pocas fuentes     
        puts "ejecutando nam..."
        exec nam salidaTCP.nam &

        close $trace_rtt
	close $trace_srtt
        close $trace_arribos
        close $trace_perdidos
        close $ventana2
        close $trace_seq2
        
	exec xgraph salidaRTT.tr salidaRTTAvg.tr -geometry 800x400 &
	exec xgraph salidaVentana2.tr -geometry 800x400 &
	exec xgraph tamCola01.tr -geometry 800x400 &
	exec xgraph salidaSeq2.tr -geometry 800x400 &
	exec xgraph paquetesPerdidos.tr paquetesArribados.tr -geometry 800x400 &

      exit 0
}

# agenda eventos

$ns at 0 "record"
$ns at 0.1   "$ftp1 start"
$ns at 80.0  "$ftp1 stop"
$ns at 80.1 "fin"

# arranca el simulador

$ns run
